/**
 Copyright (C) 2016, IAConnects Technology Limited
 All Rights Reserved

 ALL INFORMATION CONTAINED HEREIN IS, AND REMAINS
 THE PROPERTY OF IACONNECTS TECHNOLOGY LIMITED AND ITS SUPPLIERS,
 IF ANY. THE INTELLECTUAL AND TECHNICAL CONCEPTS CONTAINED
 HEREIN ARE PROPRIETARY TO IACONNECTS TECHNOLOGY LIMITED
 AND ITS SUPPLIERS AND ARE PROTECTED BY TRADE SECRET OR COPYRIGHT LAW.
 DISSEMINATION OF THIS INFORMATION OR REPRODUCTION OF THIS MATERIAL
 IS STRICTLY FORBIDDEN UNLESS PRIOR WRITTEN PERMISSION IS OBTAINED
 FROM IACONNECTS TECHNOLOGY LIMITED.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 **/

'use strict';

module.exports = function (RED) {
  function TotaliserNode (n) {
    RED.nodes.createNode(this, n);
    var node = this;
    node.total = [];
    node.startValue = [];
    node.endValue = [];
    node.startTime = null;
    node.format = 'single';
    node.output = n.output;
    node.interval = n.interval;
    if (n.intervalUnits === 'minutes') {
      node.interval = n.interval * 60;
    } else if (n.intervalUnits === 'hours') {
      node.interval = n.interval * 3600;
    }

    if (node.startTime === null) {
      node.startTime = Date.now();
    }

    updateStatus();

    node.updateTimer = setInterval(() => {
      if (node.interval - parseInt((Date.now() - node.startTime) / 1000) <= 0) {
        if (node.format === 'single') {
          let payload = {
            total: node.total.length === 0 ? 0 : node.total[0],
            startTime: new Date(node.startTime).toISOString(),
            endTime: new Date(Date.now()).toISOString()
          };
          node.startTime = Date.now();
          if (node.output === 'sum') {
            node.total = [];
          }
          if (node.output === 'diff') {
            payload.startValue = node.startValue.length === 0 ? 0 : node.startValue[0];
            payload.endValue = node.endValue.length === 0 ? 0 : node.endValue[0];
            node.startValue[0] = node.endValue[0];
            node.total = [0];
          }
          node.send({
            topic: 'total',
            payload: payload
          });
          updateStatus();
        }
        if (node.format === 'array') {
          let payload = {
            total: node.total.slice(),
            startTime: new Date(node.startTime).toISOString(),
            endTime: new Date(Date.now()).toISOString()
          };
          node.startTime = Date.now();
          if (node.output === 'sum') {
            node.total = [];
          }
          if (node.output === 'diff') {
            payload.startValue = node.startValue;
            payload.endValue = node.endValue;
            node.startValue = node.endValue;
            for (let lp = 0; lp < Math.max(node.startValue.length, node.endValue.length); lp++) {
              node.total[lp] = (node.endValue[lp] === undefined ? 0 : node.endValue[lp]) - (node.startValue[lp] === undefined ? 0 : node.startValue[lp]);
            }
          }
          node.send({
            topic: 'total',
            payload: payload
          });
          updateStatus();
        }
      }
    }, 500);

    node.on('input', function (msg) {
      if ('payload' in msg) {
        if (msg.payload === 'reset') {
          node.startTime = Date.now();
          node.total = [];
          node.startValue = [];
          node.endValue = [];
        } else {
          if (typeof msg.payload === 'number') {
            node.format = 'single';
            handleSingleInput(msg.payload);
          }
          if (Array.isArray(msg.payload)) {
            node.format = 'array';
            handleArrayInput(msg.payload);
          }
        }
        // } else if (node.format === 'single') {
        //   handleSingleInput(msg.payload);
        // } else if (node.format === 'array') {
        //   handleArrayInput(msg.payload);
        // }
      }
      updateStatus();
    });

    node.on('close', function () {
      if (this.updateTimer !== null) {
        clearInterval(this.updateTimer);
      }
    });

    function handleSingleInput (payload) {
      if (typeof payload === 'number') {
        if (node.output === 'total' || node.output === 'sum') {
          if (node.total.length === 0) {
            node.total[0] = payload;
          } else {
            node.total[0] += payload;
          }
        }
        if (node.output === 'diff') {
          if (node.startValue.length === 0) {
            node.startValue[0] = payload;
          }
          node.endValue[0] = payload;
          node.total[0] = node.endValue[0] - node.startValue[0];
        }
      }
    }

    function handleArrayInput (payload) {
      if (Array.isArray(payload)) {
        for (let lp = 0; lp < payload.length; lp++) {
          if (typeof payload[lp] !== 'number') {
            return;
          }
        }
        if (node.output === 'total' || node.output === 'sum') {
          if (node.total.length === 0) {
            node.total = payload;
          } else {
            for (let lp = 0; lp < payload.length; lp++) {
              node.total[lp] = (node.total[lp] === undefined ? 0 : node.total[lp]) + payload[lp];
            }
          }
        }
        if (node.output === 'diff') {
          if (node.startValue.length === 0) {
            node.startValue = payload;
          }
          node.endValue = payload;
          for (let lp = 0; lp < payload.length; lp++) {
            node.total[lp] = (node.endValue[lp] === undefined ? 0 : node.endValue[lp]) - (node.startValue[lp] === undefined ? 0 : node.startValue[lp]);
          }
        }
      }
    }

    function updateStatus () {
      if (node.output === 'diff') {
        node.status({
          fill: 'green',
          shape: 'dot',
          text: `Start: ${node.startValue[0] === undefined ? 0 : node.startValue[0]}, End: ${node.endValue[0] === undefined ? 0 : node.endValue[0]}`
        });
      } else {
        node.status({
          fill: 'green',
          shape: 'dot',
          text: `Total: ${node.total[0] === undefined ? 0 : node.total[0]}`
        });
      }
    }
  }

  RED.nodes.registerType('totaliser', TotaliserNode);
};
